
#include <stdlib.h>
#include "cart.h"
#include "mbc/romonly.h"

static void dummy_reset();

void (*mbc_reset)() = dummy_reset;

int cart_load(const char *fname, cart_info *info)
{
  unsigned char typeid, romsize, ramsize;
  int res;
  FILE *fd;

  fd = fopen(fname, "rb");
  if (!fd) {
    printf("%s: failure reading '%s' for reading!\n", __FILE__, fname);
    return 0;
  }
  
  /* Read the title */
  fseek(fd, 0x134, SEEK_SET);
  fread(info->title, 16, 1, fd);

  /* Read the cart type */
  fseek(fd, 0x147, SEEK_SET);
  fread(&typeid, 1, 1, fd);

  /* Read the ROM size */
  fread(&romsize, 1, 1, fd);
  if (romsize <= 7) {
    info->romsize = (32 << romsize)*1024;
  }
  else {
    switch(romsize) {
    case 52: info->romsize = 72 * 0x4000; break;
    case 53: info->romsize = 80 * 0x4000; break;
    case 54: info->romsize = 96 * 0x4000; break;
    }
  }

  /* Read the RAM size */
  fread(&ramsize, 1, 1, fd);
  switch(ramsize) {
  case 0x00: info->ramsize=0;
  case 0x01: info->ramsize=0x400*2;
  case 0x02: info->ramsize=0x400*8;
  case 0x03: info->ramsize=0x400*32;
  }

  info->rombanks = malloc(info->romsize);
  memset(info->rombanks, 0, info->romsize);

  if (info->ramsize) {
    info->rambanks = malloc(info->ramsize);
    memset(info->rambanks, 0, info->ramsize);
  }
  else {
    info->rambanks=NULL;
  }

  /* Read the ROM banks */
  fseek(fd, 0, SEEK_SET);
  fread(info->rombanks, 1, info->romsize, fd);

  /* Read the RAM banks */
/*   if (info->rambanks) fread(info->rambanks, 1, info->ramsize, fd); */

  /* Close the file */
  fclose(fd);


  switch(typeid) {
  case CART_ROM_ONLY:
    info->typeid = CART_ROM_ONLY;
    res = romonly_load(info);
    break;

  case CART_MBC1:
    info->typeid = CART_MBC1;
    res = mbc1_load(info);
    break;

  case CART_MBC1_RAM:
    info->typeid = CART_MBC1_RAM;
    res = mbc1_load(info);
    break;

  default:
    printf("%s: MBC #%.2x is not yet supported\n", __FILE__, typeid);
    res = 0;
    break;
  }

  return res;
}

void cart_cleanup(cart_info *info)
{
  free(info->rombanks);
  if (info->rambanks) free(info->rambanks);
}

static void dummy_reset()
{
}
