
#ifndef CART_H
#define CART_H

#include "mem.h"

/* Cart type constants */
#define CART_ROM_ONLY                   0x00
#define CART_MBC1                       0x01
#define CART_MBC1_RAM                   0x02
#define CART_MBC1_RAM_BATTERY           0x03
#define CART_MBC2                       0x04
#define CART_MBC2_BATTERY               0x05
#define CART_ROM_RAM                    0x06
#define CART_ROM_RAM_BATTERY            0x07
#define CART_MMM01                      0x0B
#define CART_MMM01_RAM                  0x0C
#define CART_MMM01_RAM_BATTERY          0x0D
#define CART_MBC3_TIMER_BATTERY         0x0F
#define CART_MBC3_TIMER_RAM_BATTERY     0x10
#define CART_MBC3                       0x11
#define CART_MBC3_RAM                   0x12
#define CART_MBC3_RAM_BATTERY           0x13
#define CART_MBC4                       0x15
#define CART_MBC4_RAM                   0x16
#define CART_MBC4_RAM_BATTERY           0x17
#define CART_MBC5                       0x19
#define CART_MBC5_RAM                   0x1A
#define CART_MBC5_RAM_BATTERY           0x1B
#define CART_MBC5_RUMBLE                0x1C
#define CART_MBC5_RUMBLE_RAM            0x1D
#define CART_MBC5_RUMBLE_RAM_BATTERY    0x1E
#define CART_POCKET_CAMERA              0xFC
#define CART_BANDAI_TAMA5               0xFD
#define CART_HuC3                       0xFE
#define CART_HuC1_RAM_BATTERY           0xFF

/* Hold information about a loaded cartridge */
typedef struct 
{
  char title[16];                    /* Cartridge title */
  unsigned char *rombanks;           /* ROM Bank data */
  unsigned char *rambanks;           /* RAM Bank data */
  unsigned int romsize;              /* Size of ROM */
  unsigned int ramsize;              /* Size of RAM */
  int typeid;                        /* Numeric ID of cart type */
  int battery;                       /* Set to 1 if external RAM is battery backed */
} cart_info;

/* Pointer to a function to reset an MBC */
extern void (*mbc_reset)();

/* Load a cartridge */
int cart_load(const char *fname, cart_info *info);

/* Cleanup a cartridge, saves battery backed RAM */
void cart_cleanup(cart_info *info);

#endif /* CART_H */
