/*

GooBer Portable Gameboy Emulator
Copyright (C) 2002 Ian Smith

This Program Is Free Software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef __LCDC_H__
#define __LCDC_H__

#include "allegro.h"

#define INT_LCDC_STAT   0x0048
#define INT_LCDC_VBLANK 0x0040

#define MODE_MASK         3

#define MODE_HBLANK          0
#define MODE_VBLANK          1
#define MODE_OAMTRANSFER     2
#define MODE_DATATRANSFER    3 

#define CYCLES_HBLANK        207
#define CYCLES_VBLANK        4560
#define CYCLES_OAMTRANSFER   80
#define CYCLES_DATATRANSFER  169

/* #define CYCLES_HBLANK        201 */
/* #define CYCLES_VBLANK        4560 */
/* #define CYCLES_OAMTRANSFER   77 */
/* #define CYCLES_DATATRANSFER  169 */


/* Holds the buffer to render to */
extern BITMAP *lcdc_buffer;

/* Holds the frame counter */
extern unsigned int lcdc_frames;

/* Flags when to redraw the LCDC display */
extern int lcdc_refresh;

/* Initialize the LCDC */
void lcdc_init();

/* Set the X/Y offset to blit the LCDC screen buffer to */
void lcdc_offset(int, int);

/* Set the width/height of the LCDC display */
void lcdc_size(int, int);

/* Reset the LCDC */
void lcdc_reset();

/* Perform any cleanup operations */
void lcdc_cleanup();

/* Perform a LCDC mode transition */
void lcdc_trans(unsigned long cycles);

/* Change the LCDC mode */
void lcdc_change(unsigned char m);

/* Check for LYC Co-incidence */
void lcdc_lyc_compare();

/* Render a scanline */
void lcdc_render_scanline();

/* Render whole display */
void lcdc_render();

/* Return accessibility of VRAM and OAM */
int can_access_vram();
int can_access_oam();

/* Return the LCDC mode */
unsigned char lcdc_mode();

/* Return the current scanline */
unsigned char lcdc_scanline();

/* Save the current display to a PCX file */
void lcdc_capture(char *fname);

#endif





















