
#include "timer.h"
#include "intr.h"

#define CYCLES_SPEED_0    1024
#define CYCLES_SPEED_1    16
#define CYCLES_SPEED_2    64
#define CYCLES_SPEED_3    256

static unsigned char divider;           /* FF04 */
static unsigned char counter;           /* FF05 */
static unsigned char modulo;            /* FF06 */
static unsigned char control;           /* FF07 */

static unsigned int cycle_count=0;
static unsigned int current_speed=CYCLES_SPEED_0;

void timer_init()
{
  timer_reset();
}

void timer_reset()
{
  divider = 0;
  counter = 0;
  modulo = 0;
  control = 0;
}

void timer_cleanup()
{
}

void timer_update(int cycles)
{
  if ((control&4)==4) {
    cycle_count+=cycles;

    if (cycle_count >= current_speed) {
      counter++;

      if (counter == 0) {
	counter = modulo;
	intr_request(INTR_TIMER);
      }
    }

    if (cycle_count >= 256) {
      divider++;
    }

  }

}

unsigned char timer_read_reg(unsigned char reg)
{
  switch (reg) {
  case 0x04:
    return divider;
  case 0x05:
    return counter;
  case 0x06:
    return modulo;
  case 0x07:
    return control;
  default:
    return 0xFF;
  }
}

void timer_write_reg(unsigned char reg, unsigned char value)
{
  switch(reg) {
  case 0x04:
    divider = 0;
    break;
  case 0x05:
    counter = value;
    break;
  case 0x06:
    modulo = value;
    break;
  case 0x07:
    control = value;
    switch(control&3) {
    case 0:
      current_speed = CYCLES_SPEED_0;
      break;
    case 1:
      current_speed = CYCLES_SPEED_1;
      break;
    case 2:
      current_speed = CYCLES_SPEED_2;
      break;
    case 3:
      current_speed = CYCLES_SPEED_3;
      break;
    }
    break;
  }
}
